<?php
namespace WebFacing\cPanel\Email;

/**
 * Exit if accessed directly
 */
if ( ! \class_exists( '\WP' ) ) {
	exit;
}

class Contact extends Plugin {

	private   static /*string*/ $contact_file = '';

	public    static function init() {
	}

	public    static function admin_init() {

		self::$contact_file = '/' . \explode( '/', \ABSPATH )[1] . '/' . \explode( '/', \ABSPATH )[2] . '/.cpanel/contactinfo';

		\add_action( 'admin_menu', function() {
			\add_submenu_page(
				self::$pf . 'accounts',
				\_x( 'cPanel® Contact Information', 'Page Title', self::$text_domain ),
				\_x( 'Contact Information', 'Submenu Label', self::$text_domain ),
				'manage_options',
				self::$pf . 'contact',
				function() {
					$editable = \is_writeable( self::$contact_file );
					if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
						$action  = \sanitize_text_field( $_POST['action'] ?? '' );
						$email   = \sanitize_email( $_POST['email'] ?? '' );
						if ( ! empty( $email ) ) {
							if ( $action === 'primary' ) {
								self::notice( $action, self::update_contact( $email ) );
							} elseif ( $action === 'secondary' ) {
								self::notice( $action, self::update_contact( $email, false ) );
							}
						}
					}

?>
		<div class="wrap">
			<style>
				.form-table {
					width: calc(30% + 32ch);
				}
				.form-table th, .form-table td {
					padding: 0 1ch;
				}
				input {
					color: initial !important;
				}
				.form-table tbody td input:focus:invalid {
					background-color: LightYellow;
					border-color: Red;
				}
			</style>
			<h1 class="wp-heading-inline"><?php echo \get_admin_page_title(); ?></h1>
			<hr class="wp-header-end" />
			<p><br /></p>
			<h2><?php echo \__( 'Primary Contact Email', self::$text_domain ); ?> ⚠️️</h2>
			<form id="edit-contact-1" name="email" method="post" action="">
				<input type="hidden" name="action" value="primary">
				<table class="form-table">
					<tr>
						<th><label for="email"><?php echo \__( 'To Email', self::$text_domain ); ?></label></th>
					</tr><tr>
						<td><input type="email" id="email"<?php echo $editable ? ' name="email" autocomplete="on" required="required"' : ' disabled="disabled"'; ?> value="<?php echo UAPI::get_contact_info(); ?>" /></td>
						<td><button type="submit" class="button-primary"<?php echo $editable ? '' : ' disabled="disabled"' ; ?>><?php echo \_x( 'Update', 'Button', self::$text_domain ); ?></button></td>
					</tr>
				</table>
			</form>
			<p><?php echo \__( '&mdash; OR &mdash;', self::$text_domain ); ?></p>
			<h2><?php echo \__( 'Secondary Contact Email', self::$text_domain ); ?> ⚠</h2>
			<form id="edit-contact-2" name="second_email" method="post" action="">
				<input type="hidden" name="action" value="secondary">
				<table class="form-table">
					<tr>
						<th><label for="email"><?php echo \__( 'To Email', self::$text_domain ); ?></label></th>
					</tr><tr>
						<td><input type="email" id="email" name="email"<?php echo $editable ? ' name="email" autocomplete="on" required="required"' : ' disabled="disabled"'; ?> value="<?php echo UAPI::get_contact_info( false ); ?>" /></td>
						<td><button type="submit" class="button-primary"<?php echo $editable ? '' : ' disabled="disabled"' ; ?>><?php echo \_x( 'Update', 'Button', self::$text_domain ); ?></button></td>
					</tr>
				</table>
			</form>
			<?php if ( true || ! $editable ) { ?>
			<hr />
			<p><?php _e( 'Sorry. These emails cannot be updated because of current cPanel® UAPI limitations.', self::$text_domain ); ?></p>
			<?php } ?>
		</div>
<?php
			}, 3 );
		} );
	}

	private static function update_contact( string $email, bool $primary = true ): \WP_Error {
		$result = new \WP_Error;
		$sep = ': ';
		$ndl = '"';
		$vdl = "'";
		$file_name     = self::$contact_file;// . '.orig';
		$lines = \file( $file_name );
		$changes = 0;
		foreach ( $lines as $i => $line ) {
			if ( \substr( $line, 0, 3 ) === '---' ) {
				$new_file = $line;
			} else {
				$parts = \explode( $sep , $line, 2 );
				$name  = \trim( $parts[0], $ndl . \PHP_EOL );
				$value = \trim( $parts[1], $vdl . \PHP_EOL );
				if ( $primary && $name == 'email' && $value !== $email ) {
					$value = $email;
					$changes++;
				}
				if ( ! $primary && $name == 'second_email' && $value !== $email ) {
					$value = $email;
					$changes++;
				}
				$new_file .= $ndl . $name . $ndl . $sep . ( \is_numeric( $value ) ? $value : $vdl . $value . $vdl ) . \PHP_EOL;
			}
		}
		if ( $changes ) {
			if ( \count( $lines ) ) {
				$bytes = \file_put_contents( $file_name, $new_file, \LOCK_EX );
				if ( $bytes < 64 ) {
					$result->add( $bytes, \_x( 'Error writing to contact data file. Please try from cPanel®.', 'Error message', self::$text_domain ) );
				} else {
					self::delete_transients();
				}
			} else {
				$result->add( 1, \_x( 'Contact data file was empty. Please try from cPanel®.', 'Error message', self::$text_domain ) );
			}
		} else {
			$result->add( $changes, \_x( 'No changes made.', 'Warning message', self::$text_domain ) );
		}
		return $result;
	}

	private static function notice( string $action, \WP_Error $result ) {
		if ( $result->has_errors() ) {
?>
				<div class="notice notice-error is-dismissible">
					<p><?php \printf(
						\_x(
							'%1$s',
							'Notice error message',
							self::$text_domain
						),
						\esc_html( $result->get_error_message() )
					); ?></p>
				</div>
<?php
		} else {
?>
				<div class="notice notice-success is-dismissible">
					<p><?php \printf(
						\_x(
							'%1$s contact email updated. NOTE: This change will not have any effect until saved again in cPanel®.',
							'Notice success message, %1$s = contact email type',
							self::$text_domain
						),
						\esc_html( \ucfirst( Plugin::email_to_utf8( $action ) ) ) );
					?></p>
				</div>
<?php
		}
	}
}
