<?php
namespace WebFacing\cPanel\Email;

/**
 * Exit if accessed directly
 */
if ( ! \class_exists( '\WP' ) ) {
	exit;
}

class New_Email extends Plugin {

	public    static /*int*/ $tz_offset;

	public    static function init() {
	}

	public    static function admin_init() {

		self::$tz_offset = \wp_timezone()->getOffset( new \DateTime );

		\add_action( 'admin_menu', function() {
			\add_submenu_page(
				self::$pf . 'accounts',
				\_x( 'cPanel® Add New Email Address', 'Page Title', self::$text_domain ),
				\_x( 'Add New Email', 'Submenu Label', self::$text_domain ),
				'manage_options',
				self::$pf . 'new-email',
				function() {
					$mail_domains = UAPI::mail_domains();
					$dest_emails = \wp_list_pluck( \array_filter( (array)UAPI::email_forwarders(), function( $account ) {
						return \is_email( $account->dest );
					} ), 'dest' );
					$fwd_emails  = \wp_list_pluck( \array_filter( (array)UAPI::email_forwarders(), function( $account ) {
						return \is_email( $account->forward );
					} ), 'forward' );
					$to_emails = \array_unique( \array_merge( \wp_list_pluck( (array)UAPI::email_accounts(), 'email' ), $dest_emails, $fwd_emails, [ \wp_get_current_user()->user_email ], [ \get_bloginfo( 'admin_email' ) ], [ \sanitize_email( UAPI::main_email_account()->email ) ] ) );

					$action = \sanitize_text_field( $_GET['action'] ?? '' );
					$type   = \sanitize_text_field( $_GET['type'  ] ?? '' );
					$email  = \sanitize_email(      $_GET['email' ] ?? '' );
					$edit   = $action === 'row-edit' && $type === 'responder' && \is_email( $email );
					if ( $edit ) {
						$responder = UAPI::email_responder( $email );
						$responder->start_date = \date_i18n( 'Y-m-d', $responder->start );
						$responder->start_time = \date_i18n( 'H:i',   $responder->start );
						$responder->stop_date  = \date_i18n( 'Y-m-d', $responder->stop  );
						$responder->stop_time  = \date_i18n( 'H:i',   $responder->stop  );
						$to_emails = \array_unique( \array_merge( $to_emails, [ \sanitize_email( $responder->from ) ] ) );
					} else {
						$responder = new \stdClass;
						$responder->from       = '';
						$responder->subject    = 'Re: %subject%';
						$responder->from       = '';
						$responder->body       = \sprintf(
							_x( 'Hello %%from%% %%email%%,


--
Regards,
Us at %1$s',
								'Email Body Suggestion, %1$s = Site Name',
								self::$text_domain
							),
							\get_bloginfo()
						);
						$responder->start_date = \date_i18n( 'Y-m-d' );
						$responder->start_time = \date_i18n( 'H:i' );
						$responder->stop_date  = \date_i18n( 'Y-m-d', \strtotime( '+1 weeks', \current_time( 'timestamp' ) ) );
						$responder->stop_time  = \date_i18n( 'H:i' );
						$responder->interval   = 24;
						$responder->is_html    = 0;
					}
?>
		<div class="wrap">
			<style>
				.form-table {
					width: 120ch;
				}
				.form-table th, .form-table td {
					padding: 0 1ch;
				}
				.form-table tbody th:nth-of-type(1), .form-table tbody td:nth-of-type(1) {
					width: 15%;
					white-space: nowrap;
					padding-right: 0;
					text-align: right;
				}
				.form-table tbody td input, .form-table tbody td select {
					width: calc( 100% - 2ch);
				}
				.form-table tbody td input:focus:invalid {
					background-color: LightYellow;
					border-color: Red;
				}
				.form-table thead th:nth-of-type(2), .form-table tbody td:nth-of-type(2) {
					width: 25%;
					padding-left: 0;
				}
				.form-table th:nth-of-type(3), .form-table td:nth-of-type(3) {
					width: 30%;
				}
				.form-table th:last-of-type, .form-table td:last-of-type {
					text-align: left;
				}
			</style>
			<h1 class="wp-heading-inline"><?php echo \get_admin_page_title(); ?></h1>
			<hr class="wp-header-end" />
			<p><br /></p>
			<h2>↪ <?php \_ex( 'New Forwarder', 'Form Heading', self::$text_domain ); ?></h2>
			<form id="new-forward" name="new-forward" method="post" action="">
				<input type="hidden" name="action" value="new-forward">
				<table class="form-table">
					<thead>
						<tr>
							<th scope="col">
								<label for="forward-user"><?php \_ex( 'From Email', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
							<th scope="col">
								<label for="forward-to"><?php \_ex( 'To Emails <small>(comma separated)</small>', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<!--input type="email" id="forward-email" name="email" autocomplete="on" required="required" /-->
								<input type="text" id="forward-user" name="user" class="user" autocomplete="on" required="required" pattern="[a-zA-Zs0-9._%+-]{2,}" />@
							</td>
							<td>
								<select id="forward-domain" name="domain" required="required">
<?php
								foreach ( $mail_domains as $mail_domain ) {
?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								}
?>
							</td>
							<td>
								<input type="text" id="forward-to" name="forward" autocomplete="on" required="required" />
							</td>
							<td>
								<button type="submit" class="button-primary"><?php \_ex( 'Add Forwarder', 'Button', self::$text_domain ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
			<form id="default-forward" name="default-forward" method="post" action="">
				<input type="hidden" name="action" value="default-forward">
				<table class="form-table">
					<thead>
						<tr>
							<th></th>
							<th scope="col">
								<label for="default-forward-domain"><?php \_ex( 'From Domain', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th scope="col">
								<label for="forward"><?php \_ex( 'To Email', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td class="default">*@</td>
							<td>
								<select id="default-forward-domain" name="domain" required="required">
<?php
								foreach ( $mail_domains as $mail_domain ) {
?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								}
?>
								</select>
							</td>
							<td>
								<input type="email" id="forward" name="forward" autocomplete="on" required="required" />
							</td>
							<td>
								<button type="submit" class="button-primary"><?php \_ex( 'Add Default Forwarder', 'Button', self::$text_domain ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
			<p><?php \_e( '&mdash; OR &mdash;', self::$text_domain ); ?></p>
			<h2>🚫 <?php \_ex( 'New Failure', 'Form Heading', self::$text_domain ); ?></h2>
			<form id="new-failure" method="post" action="">
				<input type="hidden" name="action" value="new-failure">
				<table class="form-table">
					<thead>
						<tr>
							<th scope="col">
								<label for="fail-user"><?php \_x( 'For Email', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
							<th scope="col">
								<label for="fail-msg"><?php \_ex( 'Failure Message', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<input type="text" id="fail-user" name="user" autocomplete="on" required="required" pattern="[a-zA-Zs0-9._%+-]{2,}" />@
							</td>
							<td>
								<select id="fail-domain" name="domain" required="required">
<?php
								foreach ( $mail_domains as $mail_domain ) {
?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								}
?>
								</select>
							</td>
							<td>
								<input type="text" id="fail-msg" name="msg" autocomplete="off" placeholder="No Such User Here" />
							</td>
							<td>
								<button type="submit" class="button-primary" name="type" value="email"><?php \_ex( 'Add Failure', 'Button', self::$text_domain ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
			<form id="default-fail" name="default-fail" method="post" action="">
				<input type="hidden" name="action" value="default-fail">
				<table class="form-table">
					<thead>
						<tr>
							<th></th>
							<th scope="col">
								<label for="default-fail-domain"><?php \_ex( 'For Domain', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th scope="col">
								<label for="default-fail-msg"><?php \_ex( 'Failure Message', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>*@</td>
							<td>
								<select id="default-fail-domain" name="domain" required="required">
<?php
								foreach ( $mail_domains as $mail_domain ) {
?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								}
?>
								</select>
							</td>
							<td>
								<input type="text" id="default-fail-msg" name="msg" autocomplete="off" placeholder="No Such User Here" />
							</td>
							<td>
								<button type="submit" class="button-primary"><?php \_ex( 'Add Default Failure', 'Button', self::$text_domain ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
			<p><?php \_e( '&mdash; OR &mdash;', self::$text_domain ); ?></p>
			<h2>⚫ <?php \_ex( 'New Blackhole', 'Form Heading', self::$text_domain ); ?></h2>
			<form id="new-black" method="post" action="">
				<input type="hidden" name="action" value="new-black">
				<table class="form-table">
					<thead>
						<tr>
							<th scope="col">
								<label for="black-user"><?php \_ex( 'For Email', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
							<th></th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<input type="text" id="black-user" name="user" placeholder="noreply" autocomplete="on" required="required" pattern="[a-zA-Zs0-9._%+-]{2,}" />@
							</td>
							<td>
								<select id="black-domain" name="domain" required="required">
<?php
								foreach ( $mail_domains as $mail_domain ) {
?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								}
?>
								</select>
							</td>
							<td></td>
							<td>
								<button type="submit" class="button-primary" name="type" value="email"><?php \_ex( 'Add Blackhole', 'Button', self::$text_domain ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
			<form id="default-black" name="default-black" method="post" action="">
				<input type="hidden" name="action" value="default-black">
				<table class="form-table">
					<thead>
						<tr>
							<th></th>
							<th scope="col">
								<label for="default-black-domain"><?php \_ex( 'For Domain', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>*@</td>
							<td>
								<select id="default-black-domain" name="domain" required="required">
<?php
								foreach ( $mail_domains as $mail_domain ) {
?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								}
?>
								</select>
							</td>
							<td></td>
							<td>
								<button type="submit" class="button-primary"><?php \_ex( 'Add Default Blackhole', 'Button', self::$text_domain ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
			<p><?php \_e( '&mdash; OR &mdash;', self::$text_domain ); ?></p>
			<h2>✉ <?php \_ex( 'New Account', 'Form Heading', self::$text_domain ); ?></h2>
			<form id="new-account" method="post" action="">
				<input type="hidden" name="action" value="new-account">
				<table class="form-table">
					<thead>
						<tr>
							<th scope="col">
								<label for="new-email"><?php \_ex(' Email Address', 'Form Field Label', self::$text_domain ); ?>
							</th>
							<th></th>
							<th scope="col">
								<label for="password"><?php \_ex( 'Account Password', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th></th>
						</tr>
					</thead>
					<tbody>
						<tr>
							<td>
								<input type="text" id="new-email" name="user" autocomplete="on" required="required" pattern="[a-zA-Zs0-9._%+-]{2,}" />@
							</td>
							<td>
								<select id="email-domain" name="domain" required="required">
<?php
								foreach ( $mail_domains as $mail_domain ) {
?>
									<option value="<?php echo $mail_domain; ?>"><?php echo \idn_to_utf8( $mail_domain ); ?></option>
<?php
								}
?>
								</select>
							</td>
							<td>
								<input type="password" id="password" name="password" pattern=".{8,}" autocomplete="new-password" required="required" title="<?php \_ex( 'At least 8 characters', 'Title text', self::$text_domain ); ?>" />
							</td>
							<td>
								<button type="submit" class="button-primary"><?php \_ex( 'Create New Account', 'Button', self::$text_domain ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
			<p><?php \_e( '&mdash; OR &mdash;', self::$text_domain ); ?></p>
			<h2>↩️ <?php if ( $edit ) \_ex( 'Edit Autoresponder', 'Form Heading', self::$text_domain ); else \_ex( 'New Autoresponder', 'Form Heading', self::$text_domain ); ?></h2>
			<form id="<?php echo $edit ? 'edit' : 'new'; ?>-responder" method="post" action="">
				<input type="hidden" name="action" value="<?php echo $edit ? 'edit' : 'new'; ?>-responder" />
				<table class="form-table" style="width: 70%">
					<thead>
						<tr>
							<th colspan="1" scope="row" style="width: 20%;">
								<label for="responder-for"><?php \_ex( 'For Emails to', 'Form Field Label', self::$text_domain ); ?></label>
							</td>
							<td colspan="4" style="width: 25%; padding-left: 0;">
							<?php if ( $edit ) { ?>
								<input id="responser-for" name="email" readonly="readonly" value="<?php echo self::email_to_utf8( $email ); ?>" />
							<?php } else { ?>
								<select id="responder-for" name="email" required="required">
									<option value="<?php $account = UAPI::main_email_account(); echo $account->email; ?>"><?php echo self::email_to_utf8( $account->email ); ?></option>
									<?php foreach ( (array)UAPI::email_accounts() as $account ) { ?>
									<option value="<?php echo $account->email; ?>"><?php echo self::email_to_utf8( $account->email ); ?></option>
									<?php } ?>
								</select>
							<?php } ?>
							</td>
							<td colspan="1"></td>
						</tr>
					</thead>
					<tbody>
						<tr>
							<th colspan="2" scope="col" style="width: 10%; text-align: left;">
								<label for="responder-from"><?php \_ex( 'From Email', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th colspan="3" scope="col" style="width: 15%;">
								<label for="responder-subject"><?php \_ex( 'Subject', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th colspan="1"></th>
						</tr>
						<tr>
							<td colspan="2" style="width: 10%; text-align: left;">
								<!--input id="responder-from" type="text" name="from" value="<?php //echo wp_get_current_user()->user_email; ?>" style="width: 28ch;" autocomplete="on" required="required" /-->
								<select id="responder-from" name="from">
								<?php foreach ( $to_emails as $to_email ) { ?>
									<option value="<?php echo $to_email; ?>"<?php \selected( $to_email, $responder->from ); ?>><?php echo self::email_to_utf8( $to_email ); ?></option>
								<?php } ?>
								</select>
							</td>
							<td colspan="3">
								<input id="responder-subject" type="text" name="subject" value="<?php echo $responder->subject; ?>" style="width: 38ch;" placeholder="Re: %subject%" autocomplete="off" required="required" />
							</td>
							<td colspan="1"></td>
						</tr>
						<tr>
							<th colspan="2" scope="col" style="width: 10%; text-align: left;">
								<label for="responder-start"><?php \_ex( 'Start time', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th colspan="2" scope="col">
								<label for="responder-stop"><?php \_ex( 'End time', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th colspan="1" scope="col" title="<?php \_ex( 'Interval before repeated to same recipient.', 'Form Field Tooltip', self::$text_domain ); ?>">
								<label for="responder-interval"><?php \_ex( 'Interval', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<th colspan="1"></th>
						</tr>
						<tr>
							<td colspan="2" style="width: 10%; text-align: left;">
								<input id="responder-start-date" type="date" name="start-date" value="<?php echo $responder->start_date; ?>" style="width: 16ch;" autocomplete="off" required="required" />
								<input type="time" name="start-time" value="<?php echo $responder->start_time; ?>" style="width: 12ch;" />
							</td>
							<td colspan="2" style="white-space: nowrap;">
								<input id="responder-start-date" type="date" name="stop-date" value="<?php echo $responder->stop_date; ?>" style="width: 15ch;" autocomplete="off" required="required" />
								<input type="time" name="stop-time" value="<?php echo $responder->stop_time; ?>" style="width: 11ch;" />
							</td>
							<td colspan="1" style="white-space="nowrap">
								<input id="responder-interval" type="number" name="interval" value="<?php echo $responder->interval; ?>" style="width: 8ch;" autocomplete="off" required="required" title="<?php \_ex( 'Interval before repeated to same recipient.', 'Form Field Tooltip', self::$text_domain ); ?>" /> <?php \_ex( 'hours', 'Input Field Unit', self::$text_domain ); ?>
							</td>
							<td colspan="1" style="width: 25%;"></td>
						</tr>
						<tr>
							<th colspan="5" scope="col" style="text-align: left;">
								<label for="responder-body"><?php \_ex( 'Email Message', 'Form Field Label', self::$text_domain ); ?></label>
							</th>
							<td colspan="1"></td>
						</tr>
						<tr>
							<td colspan="5" style="text-align: left; padding-right: 0:">
								<textarea id="responder-body" name="body" cols="67" rows="6" required="required"><?php echo $responder->body; ?></textarea>
							</td>
							<td style="width: 25%; vertical-align: bottom;">
								<button type="submit" class="button-primary"><?php if ( $edit ) \_ex( 'Save Autoresponder', 'Button', self::$text_domain ); else \_ex( 'Create New Autoresponder', 'Button', self::$text_domain ); ?></button>
							</td>
						</tr>
					</tbody>
				</table>
			</form>
		</div>
<?php
				\add_action( 'admin_notices', function() {
					$action = \sanitize_text_field( $_GET[ 'action' ] ?? '' );
					if ( $action ) {
?>
						<div class="notice notice-success is-dismissible">
							<p><?php echo \esc_attr( \ucfirst( $action ) ); ?> created!</p>
						</div>
<?php
					}
				} );
			}, 1 );

			if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
				$action  = \sanitize_text_field( $_POST['action'] ?? '' );
				$user    = \sanitize_text_field( $_POST['user' ] ?? '' );
				$domain  = \sanitize_text_field( $_POST['domain' ] ?? '' );
				$email   = $user && $domain ? \sanitize_email( $user . '@' . $domain ) : '';
				if ( $action === 'new-forward' ) {
					$forward = \sanitize_text_field( $_POST['forward'] ?? '' );
					self::notice( $action, UAPI::add_forwarder( $email, $forward ) );
				} elseif ( $action === 'default-forward' ) {
					$forward = \sanitize_email( $_POST['forward'] ?? '' );
					self::notice( $action, UAPI::set_default_email( $domain, $forward ) );
				} elseif ( $action === 'new-failure' ) {
					$message = \sanitize_text_field( $_POST['msg'] ?? '' );
					self::notice( $action, UAPI::add_fail( $email, $message ) );
				} elseif ( $action === 'default-fail' ) {
					$message = \sanitize_text_field( $_POST['msg'] ?? '' );
					self::notice( $action, UAPI::set_default_fail( $domain, $message ) );
				} elseif ( $action === 'new-black' ) {
					self::notice( $action, UAPI::add_blackhole( $email ) );
				} elseif ( $action === 'default-black' ) {
					self::notice( $action, UAPI::set_default_blackhole( $domain ) );
				} elseif ( $action === 'new-account' ) {
					$password = \sanitize_text_field( $_POST['password'] ?? '' );
					self::notice( $action, UAPI::add_account( $email, $password ) );
				} elseif ( $action === 'new-responder' || $action === 'edit-responder' ) {
					$email     = \sanitize_email( $_POST['email' ] ?? '' );
					$from      = \sanitize_text_field( $_POST['from' ] ?? '' );
					$subject   = \sanitize_text_field( $_POST['subject' ] ?? '' );
					$body      = \sanitize_textarea_field( $_POST['body' ] ?? '' );
					$start     = \mysql2date( 'U', $_POST['start-date'] . ' ' . $_POST['start-time'] ) - self::$tz_offset;
					$stop      = \mysql2date( 'U', $_POST['stop-date' ] . ' ' . $_POST['stop-time' ] ) - self::$tz_offset;
					$interval  = \intval( $_POST['interval'] );
					self::notice( $action, UAPI::add_responder( $email, $from, $subject, $body, $start, $stop, $interval ) );
				}
				self::delete_transients();
			}
		} );
	}

	private static function notice( string $action, \WP_Error $result ) {
		if ( $result->has_errors() ) {
			\add_action( 'admin_notices', function() use ( $result ) {
				\printf( '<div class="notice notice-error"><p>%1$s</p></div>', esc_html( $result->get_error_message() ) );
			} );
		} else {
			\add_action( 'current_screen', function() use ( $action ) {
				\wp_redirect( \add_query_arg( [ 'action' => $action ], \menu_page_url( self::$pf . 'accounts' ) ) );
				exit;
			}, 0 );
		}
	}
}
