<?php
namespace WebFacing\cPanel\Email;

/**
 * Exit if accessed directly
 */
if ( ! \class_exists( '\WP' ) ) {
	exit;
}

abstract class Plugin {

	const pf = 'wf_cpanel_email_';

	protected static /*array*/      $plugin_data;

	public    static /*string*/     $plugin_name;

	public    static /*string*/     $plugin_version;

	protected static /*string*/     $domain_path;

	public    static /*string*/     $text_domain;

	public    static /*string*/     $pf;

	protected static /*?\WP_User*/  $dev;

	protected static /*bool*/       $is_debug;

	protected static /*\WP_Screen*/ $wp_screen;

	protected static /*string*/     $screen_id = '';

	protected static /*bool*/       $is_cpanel;

	protected static /*int*/        $main_parts;

	public    static /*?string*/    $host_name;

	protected static /*array*/      $transients = [];

	protected static /*string*/     $transient_time = 10 * \MINUTE_IN_SECONDS;

	public    static function init() {

		self::$host_name     = \gethostbyaddr( $_SERVER['SERVER_ADDR'] ) ?: null;

		if ( ! \function_exists( '\get_plugin_data' ) ) {
			require_once \ABSPATH . 'wp-admin/includes/plugin.php';
		}
		Table_Page::init();
		New_Email::init();
		Box_Page::init();
		Contact::init();
	}

	public    static function admin_init() {

		self::$plugin_data     = \get_plugin_data( PLUGIN_FILE );

		self::$plugin_name     = self::$plugin_data['Name'];

		self::$plugin_version  = self::$plugin_data['Version'];

		self::$domain_path     = \trailingslashit( \dirname( PLUGIN_FILE ) . self::$plugin_data['DomainPath'] );

		self::$text_domain     = self::$plugin_data['TextDomain'];
		$tx = self::$text_domain;

		self::$pf              = \trim( \str_replace( '_', '-', self::pf ) );

		self::$is_debug        = \defined( '\WF_DEBUG' ) && \WF_DEBUG;

		self::$is_cpanel       = \function_exists( '\shell_exec' ) && UAPI::has_features();

		self::$main_parts      = \count( \explode( '.', UAPI::main_domain() ) );

		\add_action( 'plugins_loaded', function() {
			\load_plugin_textdomain( self::$text_domain, false, self::$domain_path );
		} );

		\add_action( 'init', function() {
			$parts = \explode( '/', \rtrim( self::$plugin_data['AuthorURI'], ' /' ) );
			self::$dev = \get_user_by( 'login', \defined( '\WF_DEV_LOGIN' ) ? \WF_DEV_LOGIN : \end( $parts ) ) ?: null;
		} );

		\add_action( 'current_screen', function() {
			self::$wp_screen = \get_current_screen();
			self::$screen_id = self::$wp_screen ? self::$wp_screen->id : 'none';
		} );

		if ( ! self::$is_cpanel ) {
			\add_action( 'admin_notices', function() use ( $tx ) {
				if ( \current_user_can( 'activate_plugins' ) && \is_object( self::$dev ) && self::$dev->ID === \get_current_user_id() ) { ?>
			<div class="notice notice-error">
				<p><?php \printf(
						\_x( 'The <code>shell_exec</code> function is disabled, cPanel® not detected, or no email features found! You should just deactivate the "%1$s" plugin because it can\'t work on this host.', '%1$ = This Plugin Name', self::$text_domain ),
						self::$plugin_name
					); ?></p>
			</div>
<?php
				}
			} );
		} elseif ( self::$is_debug  ) {
			\add_action( 'admin_notices', function() use ( $tx ) {
				if ( \is_object( self::$dev ) && self::$dev->ID === \get_current_user_id() ) { ?>
			<div class="notice notice-success is-dismissible">
				<p>
					<?php echo self::$plugin_name, ' ', self::$plugin_version; ?><!--br /-->
					<?php //echo \plugin_basename( PLUGIN_FILE ); ?><!--br /-->
				</p>
			</div>
<?php
				}
			} );
		}
		if ( self::$is_cpanel ) {
			Table_Page::admin_init();
			New_Email::admin_init();
			Box_Page::admin_init();
			Contact::admin_init();
		}

		\add_filter( 'dashboard_glance_items', function ( ?array $elements ): array {
			$elements = $elements ?? [];
			// Protect against duplicate from my other plugin
			$dup = \preg_grep( '/ class="email-count"/', $elements );
			if ( \count( $dup ) ) {
				unset( $elements[ \array_keys( $dup)[0] ] );
			}
			if ( self::$is_cpanel && \current_user_can( 'manage_options' ) ) {
				$email_accounts = UAPI::email_accounts();
				$num_accounts = 1 + ( \is_array( $email_accounts ) ? \count( $email_accounts ) : 0 );
				$elements[ self::$pf ] = '<a href="' . \add_query_arg( [ 'page' => self::$pf . 'accounts' ], \admin_url( 'admin.php' ) ) . '" class="email-count">' . $num_accounts . ' ' . \_nx( 'email account used on cPanel&reg;', 'email accounts used on cPanel&reg;', $num_accounts, 'Right Now - suffix for number for email accounts', self::$text_domain ) . '</a>';
			}
			return $elements;
		}, 101 );

		\add_action( 'rightnow_end', function() {
			if ( \current_user_can( 'install_plugins' ) ) {
				if ( ! \class_exists( '\WebFacing\cPanel\Plugin' ) ) { ?>
					<p><small><strong><?php \printf(
						\_x( 'Note from the author of the &laquo;%1$s&raquo; plugin:',
								'%1$s = Plugin Name',
								self::$text_domain ),
							\_x( 'WebFacing – Email Accounts in cPanel®', 'Plugin Name', self::$text_domain ),
						);
						?></strong><br/><?php \printf(
						\_x( 'Also check out this complementary <a href="%2$s">&laquo;%1$s&raquo;</a> plugin.',
								'%1$ = Plugin Name, %2$s = uri (also localize)',
								self::$text_domain ),
							\_x( 'WebFacing – Disk Usage from cPanel® on Dashboard &amp; in Site Health', 'Plugin Name', self::$text_domain ),
							\_x( 'https://wordpress.org/plugins/wf-cpanel-right-now-site-health/', 'Plugin URI', self::$text_domain )
						); ?></small></p> <?php
				}
			}
		} );


		/*
		 * Custom Icon for Disk space in "At a Glance" widget
		 */
		\add_action( 'admin_head', function() {
			if ( self::$screen_id === 'dashboard' ) { ?>
				<style>
					#dashboard_right_now li a.email-count:before {
						content: '\f465';
						margin-left: -1px;
					}
				</style>
<?php
			}
		} );

		\add_action( 'admin_footer', function() {
			$transients = \get_transient( self::$pf . 'transients' );
			$transients = \array_unique( \array_merge( self::$transients, \is_array( $transients ) ? $transients : [] ) );
			\set_transient( self::$pf . 'transients', $transients, self::$transient_time );
			error_log( 'save_transients ' . \print_r( self::$transients, true ) );
		} );
	}

	public    static function delete_transients() {
		$transients = \get_transient( self::$pf . 'transients' );
		error_log( __METHOD__ . ' ' . \print_r( $transients, true ) );
		if ( \is_array( $transients ) ) {
			foreach ( $transients as $transient ) {
				\delete_transient( $transient );
			}
		}
		\delete_transient( self::$pf . 'transients' );
	}

	public    static function email_mx_self( string $domain = '' ): bool {
		$domain = $domain ?: self::$site_domain;
		$hosts = [];
		$result = \getmxrr( $domain, $hosts );
		return $result && \array_key_exists( 0, $hosts ) && \str_ends_with( $hosts[0], $domain );
	}

	public    static function email_to_utf8( string $email ): string {
		if ( \is_email( $email ) ) {
			$parts = \explode( '@', $email, 2 );
			return $parts[0] . '@' . \idn_to_utf8( $parts[1] );
		} else {
			return $email;
		}
	}
}
