<?php
namespace WebFacing\cPanel\Email;

/**
 * Exit if accessed directly
 */
if ( ! \class_exists( '\WP' ) ) {
	exit;
}

class Table_Page extends Plugin {

	private static /*string*/ $page;

	public    static function init() {
	}

	public    static function admin_init() {

		self::$page = self::$pf . 'accounts';

		\add_action( 'admin_menu', function() {
			$user_email = \wp_get_current_user()->user_email;
			$user_domain = \explode( '@', $user_email )[1];
			$site_domain = $_SERVER['SERVER_NAME'];
			$args = [ 'singular' => 'email', 'plural' => 'emails' ];
			if ( ! \current_user_can( 'manage_options' ) && \str_ends_with( $site_domain, $user_domain ) ) {
				$args['email'] = $user_email;
			}
			$role = \array_key_exists( 'email', $args ) ? 'cpanel' : 'manage_options';
			\add_menu_page( \_x( 'cPanel® Email Addresses', 'Page title', Plugin::$text_domain ), 'cPanel®', $role, self::$pf . 'accounts', null, 'dashicons-email', 27 );
			\add_submenu_page(
				self::$pf . 'accounts',
				\_x( 'cPanel® Email Addresses', 'Page title', Plugin::$text_domain ),
				\_x( 'All Email Addresses', 'Submenu label', Plugin::$text_domain ),
				$role,
				self::$pf . 'accounts',
				function() use ( $args ) {
					$list_table = new List_Table( $args );
					$list_table->prepare_items();
?>
				<style>
					.wp-list-table .column-type {
						width: 16ch;
					}
					.wp-list-table .column-diskused,
					.wp-list-table .column-diskquota {
						width: 12ch;
					}
				</style>
				<div class="wrap">
					<h1 class="wp-heading-inline"><?php echo \get_admin_page_title(); ?></h1>
					<?php if ( ! \array_key_exists( 'email', $args ) ) { ?>
					<a href="<?php echo \menu_page_url( self::$pf . 'new-email', false ); ?>" class="page-title-action"><?php  \_ex( 'Add new', 'Page Title Button', self::$text_domain ); ?></a>
					<?php } else {
						echo \__( 'Limited to', self::$text_domain ), ' <code>', Plugin::email_to_utf8( $args['email'] ), '</code>';
					} ?>
					<hr class="wp-header-end" />
					<form method="POST" action="">
						<input type="hidden" name="page" value="<?php echo self::$pf; ?>accounts">
						<?php $list_table->display(); ?>
					</form>
<?php
					if ( ! \class_exists( 'WebFacing\cPanel\Plugin' ) && \current_user_can( 'install_plugins' ) ) {
?>
					<p><br /><small><strong><?php \_e( 'Note from the plugin author:', self::$text_domain ); ?></strong> <?php \printf(
						\_x( 'Also check out this complementary <a href="%2$s">%1$s</a> plugin.', '%1$ = Plugin Name, %2$s = url (also localize)', self::$text_domain ),
						'WebFacing – Disk Usage from cPanel® on Dashboard & in Site Health',
						'https://wordpress.org/plugins/wf-cpanel-right-now-site-health/'
						); ?><a></a></small></p>
<?php
					}
?>
				</div>
<?php
				}
			);
		} );

		\add_filter( 'removable_query_args', function( array $args ): array {
			if ( \sanitize_text_field( $_GET['page'] ?? '' ) === self::$page ) {
				$args[] = 'action';
				$args[] = 'type';
				$args[] = 'email';
				$args[] = 'dest';
				$args[] = '_wpnonce';
			}
			return $args;
		} );

		\add_action( 'wp_ajax_' . self::$pf . 'password', function() {
			$method = \strtolower( $_SERVER['REQUEST_METHOD'] );
			if ( $method === 'post' && \current_user_can( 'manage_options' ) ) {
				$email = \sanitize_email( $_POST['email'] );
				$result = UAPI::set_password( $email, \sanitize_text_field( $_POST['password'] ) );
				if ( $result->has_errors() ) {
					\esc_attr_e( $result->get_error_message() );
				} else {
					\printf( __( 'Password changed for %1$s.', self::$text_domain ), $email );
				}
			}
			\wp_die();
		} );

		\add_action( 'wp_ajax_' . self::$pf . 'quota', function() {
			$method = \strtolower( $_SERVER['REQUEST_METHOD'] );
			if ( $method === 'post' && \current_user_can( 'manage_options' ) ) {
				$email = \sanitize_email( $_POST['email'] );
				$quota = \intval( $_POST['quota'] );
				$result = UAPI::set_quota( $email, $quota );
				if ( $result->has_errors() ) {
					\esc_attr_e( $result->get_error_message() );
				} else {
					\printf( __( 'Quota changed to %1$f GB for %2$s.', self::$text_domain ), $quota * \MB_IN_BYTES / \GB_IN_BYTES, $email );
					self::delete_transients();
				}
			}
			\wp_die();
		} );

		\add_action( 'admin_head', function() {
?>
			<script>
				jQuery( function() {
					jQuery( 'form.webmail' ).on( 'submit', function( event ) {
						jQuery( event.target ).find( '.button[formtarget]' ).prop( 'disabled', 'disabled' );
					} );
					setTimeout( function() {
						jQuery( 'form.webmail .button[formtarget]' ).prop( 'disabled', 'disabled' );
					}, 600000 );
				} );
			</script>
<?php
		} );
	}
}
