<?php
namespace WebFacing\cPanel\Email;

/**
 * Exit if accessed directly
 */
if ( ! \class_exists( '\WP' ) ) {
	exit;
}

class UAPI extends Plugin {

	protected static function call( string $module, string $function, ?array $params = [], string $output = 'json' ): ?string {
		$paramst = '';
		foreach( $params as $name => $value ) {
			$paramst .= " " . $name . '=' . ( \is_integer( $value ) ? $value : "'" . \urlencode( $value ) . "'" );
		}
		return \shell_exec( 'uapi --output=' . $output . ' ' . $module . ' ' . $function . $paramst );
	}

	protected static function result( string $module, string $function, array $params = [], bool $return = false )/*: \stdClass | array*/ {
		if ( ! $return ) {
			$transient_name = self::$pf . $module . '-' . $function . ( \count( $params ) ? '-' . \serialize( $params ) : '' );
			$response = \json_decode( \get_transient ( $transient_name ) );
			$data_fresh = ! (
				( \is_string( $response ) && \strlen( $response ) > 0     ) ||
				( \is_object( $response ) &&   isset( $response->result ) ) ||
				( \is_array(  $response ) &&  \count( $response ) > 0     ) ||
			false );
			if ( $data_fresh ) {
				$response_json = self::call( $module, $function, $params );
				if ( ! $return ) {
					\set_transient( $transient_name, $response_json, self::$transient_time );
					self::$transients[] = $transient_name;
				}
				$response = \json_decode( $response_json );
			}
		} else {
			$response_json = self::call( $module, $function, $params );
			$response = \json_decode( $response_json );
		}
		return $return ? $response->result : $response->result->data;
	}

	public    static function has_features( array $features = [ 'email_disk_usage', 'popaccts', 'forwarders', 'webmail' ] ): bool {
		$has = true;
		foreach ( $features as $feature ) {
			$has = $has && self::result( 'Features', 'has_feature', [ 'name' => $feature ] );
		}
		return $has;
	}

	public    static function main_domain(): string {
		return self::result( 'DomainInfo', 'list_domains' )->main_domain;
	}

	public    static function main_email_account(): \stdClass {
		$account = new \stdClass;
		$domain = self::main_domain();
		$account->domain = $domain;
		$account->email = self::result( 'Variables',  'get_user_information' )->user . '@' . $domain;
		$account->_diskused  = (int)self::result( 'Email', 'get_main_account_disk_usage_bytes' );
		$account->diskquota = '∞';
		return $account;
	}

	public    static function email_accounts( string $domain = '' ): ?array {
		$params = [ 'infinitylang' => 1/*, 'maxaccounts' => 999*/ ];
		if ( $domain ) {
			$params['domain'] = $domain;
		}
		return self::result( 'Email', 'list_pops_with_disk', $params );
	}

	public    static function email_forwarders(): ?array {
		return self::result( 'Email', 'list_forwarders' );
	}

	public    static function webmail_settings( string $email ): string {
		return self::result( 'Email', 'get_webmail_settings' )->domain;
	}

	public    static function main_disk_usage(): ?int {
		$used = self::result( 'Email', 'get_main_account_disk_usage_bytes' );
		return $used;
	}

	public    static function add_forwarder( string $from, string $to ): \WP_Error {
		$domain = \explode( '@', $from )[1];
		$result = self::result( 'Email', 'add_forwarder', [
			'domain'   => $domain,
			'email'    => $from,
			'fwdopt'   => 'fwd',
			'fwdemail' => $to,
		], true );
		$errors = new \WP_Error;
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function add_fail( string $email, string $message = '' ): \WP_Error {
		$errors = new \WP_Error;
		$domain = \explode( '@', $email, 2 )[1];
		$args   = [
			'domain'   => $domain,
			'email'    => $email,
			'fwdopt'   => 'fail',
		];
		if ( ! empty ( $message ) ) {
			$args['failmsgs'] = $message;
		}
		$result = self::result( 'Email', 'add_forwarder', $args , true );
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function add_blackhole( string $email ): \WP_Error {
		$errors = new \WP_Error;
		$domain = \explode( '@', $email )[1];
		$args   = [
			'domain'   => $domain,
			'email'    => $email,
			'fwdopt'   => 'blackhole',
		];
		$result = self::result( 'Email', 'add_forwarder', $args , true );
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function add_account( string $email, string $password ): \WP_Error {
		$user   = \explode( '@', $email )[0];
		$domain = \explode( '@', $email )[1];
		$result = self::result( 'Email', 'add_pop', [
			'email'              => $user,
			'password'           => $password,
			'domain'             => $domain,
			'send_welcome_email' => true,
		], true );
		$errors = new \WP_Error;
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function add_responder( string $email, string $from, string $subject, string $body, int $start, int $stop, int $interval ): \WP_Error {
		$user   = \explode( '@', $email )[0];
		$domain = \explode( '@', $email )[1];
		$result = self::result( 'Email', 'add_auto_responder', [
			'body'     => $body,
			'domain'   => $domain,
			'email'    => $user,
			'from'     => $from,
			'interval' => $interval,
			'is_html'  => false,
			'start'    => $start,
			'stop'     => $stop,
			'subject'  => $subject,
		], true );
		$errors = new \WP_Error;
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function delete_account( string $email ): \WP_Error {
		$domain = \explode( '@', $email )[1];
		$result = self::result( 'Email', 'delete_pop', [
			'email'    => $email,
			'domain'   => $domain,
		], true );
		$errors = new \WP_Error;
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function delete_forward( string $email, string $destinations ): \WP_Error {
		$errors = new \WP_Error;
		foreach ( \explode( ',', $destinations ) as $destination ) {
			$result = self::result( 'Email', 'delete_forwarder', [
				'address'   => $email,
				'forwarder' => $destination,
			], true );
			if ( $result->status === 0 && \is_array( $result->errors ) ) {
				foreach ( $result->errors as $code => $error ) {
					if ( ! empty( $error ) ) {
						$errors->add( $code, $error );
					}
				}
			}
		}
		return $errors;
	}

	public    static function delete_responder( string $email ): \WP_Error {
		$errors = new \WP_Error;
		foreach ( \explode( ',', $destinations ) as $destination ) {
			$result = self::result( 'Email', 'delete_auto_responder', [
				'email'   => $email,
			], true );
			if ( $result->status === 0 && \is_array( $result->errors ) ) {
				foreach ( $result->errors as $code => $error ) {
					if ( ! empty( $error ) ) {
						$errors->add( $code, $error );
					}
				}
			}
		}
		return $errors;
	}

	public    static function send_settings( string $email, string $to ): \WP_Error {
		$errors = new \WP_Error;
		$result = self::result( 'Email', 'dispatch_client_settings', [
			'account' => $email,
			'to'      => $to,
		], true );
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function create_main_webmail_session( ?string $remote_address = null ): ?\stdClass {
		return self::result( 'Session', 'create_webmail_session_for_self', [ 'remote_address' => $remote_address ?? $_SERVER['REMOTE_ADDR'] ] );
	}

	public    static function create_webmail_session( string $email, ?string $remote_address = null ): ?\stdClass {
		$user   = \explode( '@', $email )[0];
		$domain = \explode( '@', $email )[1];
		return self::result( 'Session', 'create_webmail_session_for_mail_user', [ 'domain' => $domain, 'login' => $user, 'remote_address' => $remote_address ?? $_SERVER['REMOTE_ADDR'] ] );
	}

	public    static function set_password( string $email, string $password ): \WP_Error {
		$errors = new \WP_Error;
		if ( empty( $password ) ) {
			$errors->add( 1, 'Empty password - not set' );
		} else {
			$domain = \explode( '@', $email )[1];
			$result = self::result( 'Email', 'passwd_pop', [
				'email'    => $email,
				'password' => $password,
				'domain'   => $domain,
			], true );
		}
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function set_quota( string $email, int $quota ): \WP_Error {
		$errors = new \WP_Error;
		$user   = \explode( '@', $email )[0];
		$domain = \explode( '@', $email )[1];
		$result = self::result( 'Email', 'edit_pop_quota', [
			'domain'   => $domain,
			'email'    => $user,
			'quota'    => $quota,
		], true );
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function get_contact_info( bool $primary = true ): string {
		$result = self::result( 'Variables', 'get_user_information' );
		return $primary ? $result->contact_email : $result->contact_email_2;
	}

	public    static function mail_domains(): array {
		return \array_filter( \wp_list_pluck( self::result( 'Email', 'list_mail_domains' ), 'domain' ), function( $element ) {
			return \count( \explode( '.', $element ) ) <= Plugin::$main_parts;
		} );
	}

	public    static function default_address( string $domain ): string {
		return self::result( 'Email', 'list_default_address', [ 'domain' => $domain ] )[0]->defaultaddress;
	}

	public    static function set_default_email( string $domain, $dest ): \WP_Error {
		$errors = new \WP_Error;
		$result = self::result( 'Email', 'set_default_address', [
			'domain'    => $domain,
			'fwdopt'    => 'fwd',
			'fwdemail'  => $dest,
		], true );
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( 2, $error );
				}
			}
		}
		return $errors;
	}

	public    static function set_default_fail( string $domain, $message = '' ): \WP_Error {
		$errors = new \WP_Error;
		$args   = [
			'domain'   => $domain,
			'fwdopt'   => 'fail',
		];
		if ( ! empty ( $message ) ) {
			$args['failmsgs'] = $message;
		}
		$result = self::result( 'Email', 'set_default_address', $args, true );
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function set_default_blackhole( string $domain ): \WP_Error {
		$errors = new \WP_Error;
		$result = self::result( 'Email', 'set_default_address', [
			'domain'    => $domain,
			'fwdopt'    => 'blackhole',
		], true );
		if ( $result->status === 0 && \is_array( $result->errors ) ) {
			foreach ( $result->errors as $code => $error ) {
				if ( ! empty( $error ) ) {
					$errors->add( $code, $error );
				}
			}
		}
		return $errors;
	}

	public    static function email_responders( string $domain = '' ) {
		if ( $domain ) {
			$params = [ 'domain' => $domain ];
		}
		return self::result( 'Email', 'list_auto_responders', $params );
	}

	public    static function email_responder( string $email ) {
		$params = ['email' => $email ];
		return self::result( 'Email', 'get_auto_responder', $params );
	}

	public    static function email_mailboxes( string $email = '' ): array {
		$email = empty( $email ) ? self::result( 'Variables',  'get_user_information' )->user : $email;
		$params = [ 'account' => $email ];
		return self::result( 'Mailboxes', 'get_mailbox_status_list', $params );
	}

	public    static function delete_messages( string $email, string $guid, string $query = 'savedbefore 52w' ) {
		$email = empty( $email ) ? self::result( 'Variables',  'get_user_information' )->user : $email;
		$params = [ 'account' => $email, 'mailbox_guid' => $guid, 'query' => $query ];
		return self::result( 'Mailboxes', 'expunge_messages_for_mailbox_guid', $params );
	}
}
